/* 
 * Copyright 2000 Chris G. Davis <davis@lrtw.org>
 */
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

void do_indent(int fd, int indent) {
	if( indent >30 ) {
		do_indent(fd, indent - 30);
		indent = 30;
	}
	write(fd, "                                                ", indent);
}

void handle_buf(int fd, char* buf, int len, int * had, int indent) {
	int i;
	if(*had == 0 ) {
		do_indent(fd, indent);
		*had = 1;
	}

	for(i=0; i<len; i++) {
		write(fd, &buf[i], 1);
		if( buf[i] == '\n' ) {
		  if( i == len - 1 )
			*had = 0;
		  else
			  do_indent(fd, indent);
		}
	}
}

int main(int ac, char **av) 
{ 
	int cpid;
	int ostdout;
	int ostderr;
	int pipeout[2];
	int pipeerr[2];
	int hadout = 0;
	int haderr = 0;
	int indent;

	if( (ac < 3 ) ||
	    (!av[2])) { 
		fprintf(stderr, "usage: %s \n", av[0]); 
		exit(1); 
	} 
	indent = atoi(av[1]);

	/*
	 * Duplicate stdout and stderr
	 * close original stdout and stderr
	 * open new stdout and stderr
	 *  grab data from new stdout and stderr
	 *  massage and output on original stdout and stderr
	 */

	if( (ostdout = dup( 1 ) ) == -1 ) {
		printf("dup stdout: [%d] %s\n", errno, strerror(errno) );
	}

	if( (ostderr = dup( 2 ) ) == -1 ) {
		printf("dup stderr: [%d] %s\n", errno, strerror(errno) );
	}

	if( pipe(pipeout) ) {
		printf("pipe out: [%d] %s\n", errno, strerror(errno) );
	}

	if( pipe(pipeerr) ) {
		printf("pipe err: [%d] %s\n", errno, strerror(errno) );
	}
	
	/*
	 * Now, fork, so that one of us can play with the data
	 */
	if ((cpid = fork()) < 0 ) {
		printf("Fork failed: %d %s\n", errno, strerror(errno));
		exit(-1);
	} else if ( cpid != 0 ) {
		char outbuf[1024];
		char errbuf[1024];
		int outlen;
		int errlen;
		fd_set rfds;
		int maxfd;
	        struct timeval tv;
		int rv;
		int status;
		pid_t retv;

		/*
		 * Now, grab data on pipe*[0],  and blow it out ostd*
		 */
		if( pipeout[0] > pipeerr[0] ) 
			maxfd = pipeout[0] + 1;
		else
			maxfd = pipeerr[0] + 1;

		while(1) { 
			FD_ZERO(&rfds);
			FD_SET(pipeout[0], &rfds);
			FD_SET(pipeerr[0], &rfds);
			tv.tv_sec = 0;
			tv.tv_usec = 100;


			rv = select(maxfd, &rfds, NULL, NULL, &tv);
			if( rv > 0 ) {
				outlen = -1;
				errlen = -1;
				if( FD_ISSET(pipeout[0], &rfds)) {
					outlen = read(pipeout[0], outbuf, 1024);
					handle_buf(ostdout, outbuf, outlen, &hadout, indent);
				}	
				if( FD_ISSET(pipeerr[0], &rfds)) {
					errlen = read(pipeerr[0], errbuf, 1024);
					handle_buf(ostderr, errbuf, errlen, &haderr, indent);
				}
				
			}

			retv = waitpid(cpid, &status, WNOHANG);
			if( retv == cpid ) {
				/* child exited */
				if(WIFEXITED(status)) {
					exit( WEXITSTATUS(status));
				}
			}

		}
		return 0;
	}

	/* we are child. go nuts */
	setsid();

	if( dup2(pipeout[1], 1 ) == -1 ) {
		printf("dup pipeout: [%d] %s\n", errno, strerror(errno) );
	}
	if( dup2(pipeerr[1], 2 ) == -1 ) {
		printf("dup pipeerr: [%d] %s\n", errno, strerror(errno) );
	}

	close(ostdout);
	close(ostderr);
	close(pipeout[0]);
	close(pipeerr[0]);

	execvp(av[2],av+2);
	fprintf(stderr, "Cannot execute %s: %s\n", av[1], strerror(errno)); 

	exit(1); 
} 
