/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)trigger.c 1.6      04/02/21 Chris G. Davis"

/** \file src/fsm/trigger.c
 *  \brief FSM Triggers code
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lous/lous.h"
#include "lous/logging.h"
#include "lous/fsm.h"
#include "internal/fsm.h"

fsm_trigger_t* fsm_trigger_alloc(char* tag, s32_t initial) {
  fsm_trigger_t* retval = NULL;

  LDFunc();
  LDParam(tag, == NULL, "%s", NULL );
  LDParam(initial, != initial, "%d", NULL);

  Lmalloc(retval, sizeof( fsm_trigger_t ), NULL);

  Lstrdup(retval->tag, tag, NULL, free( retval ); );
  retval->initial = initial;

  return retval;
}


fsm_trigger_t* fsm_trigger_copy(fsm_trigger_t* trigger) {
  fsm_trigger_t* retval = NULL;

  LDFunc();
  LDParam(trigger, == NULL, "%p", NULL );

  if( (retval = fsm_trigger_alloc(trigger->tag, trigger->initial)) == NULL ) {
    LError(LD_fsm, "Unable to malloc");
  }

  return retval;
}


void fsm_trigger_add(fsm_t* fsm, fsm_trigger_t* trigger ) {

  LDFunc();
  LDParam(fsm, == NULL, "%p");
  LDParam(trigger, == NULL, "%p");

  if( fsm->lock == true ) {
    LError("fsm locked.");
    errno = EPERM;
    return;
  }

  if( hash_insert( fsm->triggers, trigger->tag, trigger ) < 0 ) {
    LError("Unable to insert");
  } else {
    fsm->num_triggers++;
  }

  return;
}

fsm_event_param_t* fsm_trigger_build_param(fsm_trigger_t* trigger ) {
  fsm_event_param_t* param;

  LDFunc();
  LDParam(trigger, == NULL, "%p", NULL );

  Lmalloc(param, sizeof( fsm_event_param_t), NULL );

  Lstrdup(param->trigger_tag, trigger->tag, NULL, free( param ); );
  param->val = trigger->initial;

  return param;
}

