/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_CLI_MACRO_H
#define LOUS_SRC_INCLUDE_CLI_MACRO_H

#ident "@(#)cli-macro.h 1.2      04/02/22 Chris G. Davis"

/** \file src/include/cli-macro.h
 *  \brief CLI macro definitions
 *  \author Chris G. Davis
 *
*/
#include "lous/cli.h"

/*
 * ************************* CLI Function Macros ****************************************
 */
/* cli expander defines */
#define CLI_EXPAND_NONE        0x00000010  /**< there is no expansion with this partial token */
#define CLI_EXPAND_ONE         0x00000020  /**< there is one expansion with this partial token */
#define CLI_EXPAND_DONE        0x00000040  /**< this partial token is already expanded */
#define CLI_EXPAND_MANY        0x00000080  /**< there are many expansions with this partial token */
#define CLI_EXPAND_ERROR       0x01000000  /**< error trying to expand this token */

/* cli command defines */
#define CLI_COMMAND_SUCCESS    0x00000100  /**< command executed successfully */
#define CLI_COMMAND_HELP       0x00000200  /**< command had an error, run help */
#define CLI_COMMAND_NOTIMPL    0x02000000  /**< command not implemented */
#define CLI_COMMAND_NOTFOUND   0x04000000  /**< command not found */
#define CLI_COMMAND_ERROR      0x08000000  /**< command had an error */

/* 
 * cli reponse functions
 */

cli_response_t* cli_response_alloc(s32_t max_lines, s32_t linew);
void cli_response_destroy(cli_response_t* resp);
void cli_response_clear(cli_response_t* resp);

void cli_response_set_retcode(cli_response_t* resp, s32_t code);
void cli_response_add_string(cli_response_t* resp, char* str, ...);
void cli_response_finish_string(cli_response_t* resp);
char* cli_response_getline(cli_response_t* resp);

/*
 * CLI_FUNC, CLI_FUNC_NI, and CLI_EXPANDER
 */
#define CLI_FUNC(fname, cpath) \
void cpath##_##fname##_func(cli_response_t* resp, tokenizer_response_t* tokens) 

#define CLI_FUNC_NI(fname, cpath) \
void cpath##_##fname##_func(cli_response_t* resp, tokenizer_response_t* tokens) { \
if(0 && tokens) {} \
cli_response_set_retcode(resp, 0x02000000);\
}

#define CLI_EXPANDER(fname) \
void fname(cli_response_t* resp, char* token)


/** set the cli func return code
 * 
 */
#define cli_return(code) cli_response_set_retcode(resp, code);

/** add to cli response
 *
 */
#define cli_print(msg, ...) \
cli_response_add_string(resp, msg, ##__VA_ARGS__ )

/** add to cli response, finish this line
 *
 */
#define cli_printf(msg, ...) \
cli_response_add_string(resp, msg, ##__VA_ARGS__ );\
cli_response_finish_string(resp)

/** function param 
 *
 */
#define cli_get_param(ind) \
 tokenizer_get_token(tokens, ind -1)

#endif /* LOUS_SRC_INCLUDE_CLI_MACRO_H */
