/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_CLI_H
#define LOUS_SRC_INCLUDE_CLI_H

#ident "@(#)cli.h 1.2      04/02/22 Chris G. Davis"

/** \file src/include/cli.h
 *  \brief CLI definitions
 *  \author Chris G. Davis
 *
*/

#include "lous/tokenizer.h"
#include "lous/splay.h"
#include "lous/queue.h"

struct CLI_Item;
typedef struct CLI_Item cli_item_t;

struct CLI_User;
typedef struct CLI_User cli_user_t;

struct CLI_Command_Response;
typedef struct CLI_Command_Response cli_response_t;

cli_item_t* cli_item_alloc(cli_item_t* parent, 
			   char* tag, 
			   void (*cli_func)(cli_response_t*, tokenizer_response_t*),
			   void (*exp_func)(cli_response_t*, char*),
			   char* help_text,
			   char* reg_string,
			   s32_t level);

void cli_item_destroy(cli_item_t*);
void cli_item_add(cli_item_t* parent, char* tag, cli_item_t* item, char* help_text);
void cli_item_display(cli_item_t* item, s32_t indent);

cli_user_t* cli_user_alloc(cli_item_t* root, s32_t buflen, s32_t lines);
void cli_user_destroy(cli_user_t* user);

s32_t cli_user_run(cli_user_t* user, tokenizer_t* toker, char* string);
cli_item_t* cli_match_token(cli_item_t* root, tokenizer_response_t* resp, s32_t level, s32_t* gotto);

char* cli_user_response_getline(cli_user_t* user);

/*
 * NOTE: this should be replaced by proper cli pre-processing
 */

/* ================ Macro => Initializer Definitions =================== */
#define CLI_NEWROOT(root) \
cli_item_t* root = NULL; \
root = cli_item_alloc(NULL, NULL, NULL, NULL, NULL, NULL, 0);

#define CLI_BRANCH(btag, cpath, htext, lvl)\
}{\
  cli_item_t* cpath##_##btag;\
  cpath##_##btag = cli_item_alloc(cpath, #btag, cpath##_##btag##_func, NULL, htext, NULL, lvl); \

#define CLI_FUNCTION(btag, cpath, htext, lvl)\
} cli_item_alloc(cpath, #btag, cpath##_##btag##_func, NULL, htext, NULL, lvl); {

#define CLI_PARAM(btag, cpath, pregex, pexpander, htext, lvl)\
}{\
  cli_item_alloc(cpath, #btag, cpath##_##btag##_func, pexpander, htext, pregex, lvl);\
}{

#define CLI_PARAMBRANCH(btag, cpath, pregex, pexpander, htext, lvl)\
}{\
  cli_item_t* cpath##_##btag;\
  cpath##_##btag = cli_item_alloc(cpath, #btag, cpath##_##btag##_func, pexpander, htext, pregex,lvl); 

#endif /* LOUS_SRC_INCLUDE_CLI_H */
