/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_SPLAY_H
#define LOUS_SRC_INCLUDE_SPLAY_H

#ident "@(#)splay.h 1.7      04/02/21 Chris G. Davis"

/** \file src/include/splay.h
 *  \brief Splay Tree definitions
 *  \author Chris G. Davis
 *
 */

/*
 * Required LOUS functionality
 */
#include "lous/iterator.h"

struct LOUS_Splay_Node;
typedef struct LOUS_Splay_Node splay_node_t; /**< splay node structure */

struct LOUS_Splay_Tree;
typedef struct LOUS_Splay_Tree splay_tree_t; /**< splay tree structure */

/** Allocate a splay tree
 *
 * @param flags
 * @param keylen
 * @param comp_func
 * @param free_func
 *
 * @return the splay tree structure, or NULL
 */
splay_tree_t* splay_alloc(s32_t flags, s32_t keylen,
			  comp_t (*comp_func)(void*, void*, s32_t),
			  void (*free_func)(void*));

/** Destroy the splay tree
 *
 * @param tree
 */
s32_t splay_destroy(splay_tree_t* tree);

/** Add the key and data to the tree
 *
 * @param tree
 * @param key
 * @param data
 */
s32_t splay_insert(splay_tree_t* tree, void* key, void* data);

/** Remove the node \a key
 *
 * @param tree
 * @param key
 *
 * @return the data associated with \a key
 */
void* splay_remove(splay_tree_t* tree, void* key);

/** Delete the node \a key
 *
 * Remove the key and the data from the tree. Delete
 * the data if \a free_func is a valid functions
 *
 * @param tree
 * @param key
 */
s32_t splay_delete(splay_tree_t* tree, void* key);

/** Replace the data associated with \a key with the new data
 *
 * @param tree
 * @param key
 * @param data
 *
 * @return the old data
 */
void* splay_replace(splay_tree_t* tree, void* key, void* data);

/** Get the data associated with \a key
 *
 * @param tree
 * @param key
 *
 * @return the data associated with \a key
 */
void* splay_lookup(splay_tree_t* tree, void* key);

/** Get a list of all keys
 *
 * @param tree
 */
void** splay_getkeys(splay_tree_t* tree);

/** Get the predessor to the node \a key
 *
 * @param tree
 * @param key
 * 
 * @return the data
 */
void* splay_getpred(splay_tree_t* tree, void* key);

/** Get the successor to the node \a key
 *
 * @param tree
 * @param key
 * 
 * @return the data
 */
void* splay_getsucc(splay_tree_t* tree, void* key);

#endif /* LOUS_SRC_INCLUDE_SPLAY_H */
