/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_INCLUDE_UNIQUEID_H
#define LOUS_SRC_INCLUDE_UNIQUEID_H

#ident "@(#)uniqueid.h 1.5      04/02/21 Chris G. Davis"

/** \file src/include/uniqueid.h
 *  \brief Hand out unique 32-bit numbers
 *  \author Chris G. Davis
 *
*/

/*
 * Required LOUS functionality
 */
#include "lous/hash.h"

struct LOUS_Unique_Set;
typedef struct LOUS_Unique_Set unique_set_t;

/** 
 * Allocate a unique_set 
 * @param min minimum value in the set
 * @param max maximum value in the set
 * @return the unique_set structure
 */
unique_set_t* unique_set_alloc(s32_t min, s32_t max);

/** 
 * Destroy a unique set
 * @param set the previously allocated unique_set to destroy
 * @return nothing
 */
void unique_set_destroy(unique_set_t* set);

/** 
 * Get a new unique value from the unique_set
 * @param set the previously allocated unique_set
 * @return the new unique value
 */
s32_t unique_set_get(unique_set_t* set);

/** 
 * Return the unique to the pool
 * @param set the previously allocated unique_set
 * @param val the unique value
 * @return nothing
 */
s32_t unique_set_put(unique_set_t* set, s32_t val);

#endif /* LOUS_SRC_INCLUDE_UNIQUEID_H */
