/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_STRING_INTERNAL_PREGEX_H
#define LOUS_SRC_STRING_INTERNAL_PREGEX_H

#ident "@(#)pregex.h 1.1      04/02/21 Chris G. Davis"

/** \file src/string/internal/pregex.h
 *  \brief Partial regular expression internal definitions
 *  \author Chris G. Davis
 *
 */

struct Partial_Regex {
  s32_t magic;

};

#define PREGEX_MAGIC 0xa35bdf23  /**< internal magic value */

/*
  Supported regex tokens

  | - branch
  ^ - beginning of line
  $ - end of line
  [] - char set, or named charset
  {} - count ({num} or {min,max} forms)
  () - recursive regex

  so "33[[xdigit]]{14}" would match 3312345678901234

  33[[] -> 33[
  33[[]] -> 33[ or 33]


  compiling: pattern a|b|c|d compiles to
  SOP
   PATTERN(a) PATTERN(b) PATTERN(c) PATTERN(d)
    EOP

  while (a|b|c|d)hmmm compiles to
  SOP
   SUB
    PATTERN(a) PATTERN(b) PATTERN(c) PATTERN(d)
    PATTERN(hmmm)
     EOP

  while (a|b|c|d)*hmmm compiles to
  SOP
   SUB (min=0, max=-1)
    PATTERN(a) PATTERN(b) PATTERN(c) PATTERN(d)
    PATTERN(hmmm)
     EOP
*/

#define PR_ELEM_SOP          0x00000000 /**< start of pattern */
#define PR_ELEM_EOP          0x00000000 /**< end of pattern  */
#define PR_ELEM_BOL          0x00000000 /**< beggining of line (^) */
#define PR_ELEM_EOL          0x00000000 /**< end of line ($) */
#define PR_ELEM_PATTERN      0x00000000 /**< pattern (abc)*/
#define PR_ELEM_BRANCH       0x00000000 /**< branch */
#define PR_ELEM_SUB          0x00000000 /**< sub-pattern */
#define PR_ELEM_SET          0x00000000 /**< char set */
#define PR_ELEM_COUNT        0x00000000 /**< count modifier */

#endif /* LOUS_SRC_STRING_INTERNAL_PREGEX_H */
