/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)queue.c 1.5      04/02/21 Chris G. Davis"

/** \file src/structures/queue.c
 *  \brief Queue structure code
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lous/lous.h"
#include "lous/logging.h"
#include "lous/queue.h"
#include "internal/linkedlist.h"

linkedlist_t* queue_alloc(s32_t flags, void (*free_func)(void*)) {
  linkedlist_t* retval=NULL;

  LDFunc();

  retval = ll_alloc(flags, free_func);

  return retval;
}

s32_t queue_destroy(linkedlist_t* list) {
  s32_t retval = -1;

  LDFunc();

  retval = ll_destroy(list);

  return retval;
}

s32_t queue_push(linkedlist_t* list, void* data) {
  s32_t retval = -1;

  LDFunc();

  if( list->flags & QUEUE_LIFO ) {
      retval = ll_insert_begin(list, data);
  } else {
    retval = ll_insert_end(list, data);
  }

  return retval;
}

void* queue_pop(linkedlist_t* list) {
  void* retval = NULL;

  LDFunc();

  retval = ll_remove_begin(list);

  return retval;
}
