/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2001,2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)util.c 1.1      04/02/16 Chris G. Davis"

/** \file src/structures/util.c
 *  \brief Utility Tree code
 *  \author Chris G. Davis
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "lous/hash.h"

comp_t default_comp(void* keya, void* keyb, s32_t keylen) {
  comp_t retval;
  s32_t val;

  if( keylen == 0 ) {
    val = strcmp(keya, keyb);
    if( val < 0 ) 
      retval = lessthen;
    else if( val == 0 )
      retval = equal;
    else
      retval = greaterthen;
  } else {
    val = memcmp(keya, keyb, keylen);
    if( val < 0 ) 
      retval = lessthen;
    else if( val == 0 )
      retval = equal;
    else
      retval = greaterthen;
  }

  return retval;
}

s32_t default_hash(void* key, s32_t keylen, s32_t buckets) {
  s32_t hash = 0;
  s32_t index;
  char* ptr = key;
  s32_t reallen;

  if( keylen == 0 ) {
    reallen = strlen( key );
  } else {
    reallen = keylen;
  }

  for(index = 0; index < reallen; index++) {
    hash ^= *ptr;
    hash ^= (*ptr << 8);
    hash ^= (*ptr << 16);
    hash ^= (*ptr << 24);
    ptr++;
  }
  return hash % (buckets - 1);
}

