/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef LOUS_SRC_USER_INTERNAL_CLI_H
#define LOUS_SRC_USER_INTERNAL_CLI_H

#ident "@(#)cli.h 1.3      04/02/22 Chris G. Davis"

/** \file src/user/internal/cli.h
 *  \brief CLI internal definitions
 *  \author Chris G. Davis
 *
 */

/** The common header in each CLI entry type
 *
 */
struct CLI_Item {
  iterator_entry_t iterate;

  char* tag;

  cli_item_t* parent; /**< The parent of this scope */

  /** command to run if no more tokens */
  void (*command_func)(cli_response_t*, tokenizer_response_t*); 

  splay_tree_t* items; /**< items in this param/scope */

  char* regex_string; /**< regex matching string, only in param */
  regex_t regex; /**< compiled regex matcher, only in param */

  /** expander function, only in param. 
      all others, expander is internal, run from the list of items

      expander takes the token, and returns the possible expansion
  */
  void (*expander)(cli_response_t*, char* token); 

  char* help_text;

  s32_t level; /**< command level, for permissions. */

};

struct CLI_Command_Response {
  s32_t ret_code;

  linkedlist_t* responses;

  char* buffer; /**< used by cli_printf and friends */
  char* bptr;
  s32_t response_lines;
  s32_t line_width;

  s32_t max_line_width;
  s32_t max_response_lines;
};
 
struct CLI_User {
  s32_t max_line_width;
  s32_t max_response_lines;

  cli_item_t* cli_root; /**< the cli tree i'm using */

  tokenizer_response_t* tresp; /**< tokenizer response */
  cli_response_t* resp; /**< last response, used for paging long responses */
};

#endif /* LOUS_SRC_USER_INTERNAL_CLI_H */
