/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)dualtest.c 1.4      04/02/23 Chris G. Davis"

/** \file tests_t0002_logtest_c
 *  \brief
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>

#include "lous/lous.h"
#include "lous/logging.h"

s32_t D_class1;
s32_t D_class2;
s32_t D_class3;

s32_t DL_light;
s32_t DL_medium;
s32_t DL_heavy;

int main () {
	loggroup_t* logging;
	loguser_t* loguser;
	loguser_t* loguser1;

	s32_t ret;

        logging = loggroup_init(true);
        if( logging == NULL ) {
                printf("can't init loggroup\n");
                exit(-1);
        }

	D_class1 = loggroup_reg_class(logging, "class1");
	D_class2 = loggroup_reg_class(logging, "class2");
	D_class3 = loggroup_reg_class(logging, "class3");

	DL_light = loggroup_reg_level(logging, "light");
	DL_medium = loggroup_reg_level(logging, "medium");
	DL_heavy = loggroup_reg_level(logging, "heavy");

	loguser = loguser_init(logging, 0, NULL );
	loguser1 = loguser_init(logging, 0, NULL );

	loguser_mask(loguser, D_class1, set);
	loguser_mask(loguser, D_class3, set);
	loguser_mask(loguser1, D_class3, set);

	ret = loguser_mask(loguser, -1, get);
	printf("got back %d => [%d] %s\n", ret, errno, strerror(errno));

	loguser_level(loguser, D_class1, set, DL_heavy);
	loguser_level(loguser, D_class3, set, DL_light);
	loguser_level(loguser1, D_class2, set, DL_heavy);
	loguser_level(loguser1, D_class3, set, DL_medium);

	Debug(logging, D_class1, DL_light, "class1 light level");

	loguser_flag(loguser, LOG_STDOUT, set);
	loguser_flag(loguser, LOG_NOTIME, set);
	loguser_flag(loguser, LOG_NODATE, set);

	loguser_flag(loguser1, LOG_STDERR, set);
	loguser_flag(loguser1, LOG_NOTIME, set);
	loguser_flag(loguser1, LOG_NODATE, set);

	Debug(logging, D_class1, DL_light, "class1 light level");
	Debug(logging, D_class1, DL_medium, "class1 medium level");
	Debug(logging, D_class1, DL_heavy, "class1 heavy level");
	Debug(logging, D_class2, DL_light, "class2 light level");
	Debug(logging, D_class2, DL_medium, "class2 medium level");
	Debug(logging, D_class2, DL_heavy, "class2 heavy level");
	Debug(logging, D_class3, DL_light, "class3 light level");
	Debug(logging, D_class3, DL_medium, "class3 medium level");
	Debug(logging, D_class3, DL_heavy, "class3 heavy level");

	ret = loggroup_destroy( logging );
	printf("got back %d => [%d] %s\n", ret, errno, strerror(errno));

	loguser_destory(loguser);
	loguser_destory(loguser1);

	ret = loggroup_destroy( logging );
	printf("got back %d => [%d] %s\n", ret, errno, strerror(errno));


	return 0;
}

