/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)splay-hash-list.c 1.4      04/02/16 Chris G. Davis"

/** \file tests/t0004/splay-hash.c
 *  \brief
 *  \author Chris G. Davis 
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include "lous/hash.h"
#include "lous/splay.h"
#include "lous/iterator.h"

struct test_ent {
	iterator_entry_t iterate;

	char key[32];
	char data[32];
};

int main() {
	hash_t* hash;
        char* data;
        char buffer[64];
        FILE* dict;
        void* ret;
	struct test_ent* ent;

        LOUS_Debug_Init(LOG_STDERR | LOG_NODATE | LOG_NOTIME,0);
        LOUS_SetDebug(LD_func, LL_veryverbose);
        LOUS_SetDebug(LD_debug, LL_veryverbose);

	hash = hash_alloc(64, 0, 
		HASH_MULTIPLEX | HASH_SPLAY | LIST_KEEPFIFO, 
		NULL, NULL, NULL);

        dict = fopen("words", "r");

        printf("Add all words: ");
        while( fgets(buffer, 64, dict)>0 ) {
		ent = malloc( sizeof( struct test_ent ));
		strdup(ent->key, buffer);
		strdup(ent->data, buffer);

                hash_insert(hash, buffer, ent);
        }
        printf("done\n");
        fclose(dict);

        dict = fopen("words", "r");

        printf("Remove all words: ");
        while( fgets(buffer, 64, dict)>0 ) {
                ret = hash_remove(hash, buffer);
                if( ret == NULL) {
                        printf("%s return invalid\n", buffer);
                }
		free(ret);
        }
        printf("done\n");
        fclose(dict);

        dict = fopen("words", "r");

        printf("Add all words: ");
        while( fgets(buffer, 64, dict)>0 ) {
		ent = malloc( sizeof( struct test_ent ));
		strdup(ent->key, buffer);
		strdup(ent->data, buffer);

                hash_insert(hash, buffer, ent);
        }
        printf("done\n");
        fclose(dict);

	return 0;
}


