/*
 * Library of Useful Stuff (LOUS), a collection of useful functions
 * Copyright (C) 2002,2004 Chris G. Davis <davis@lrtw.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ident "@(#)longtest.c 1.3      04/02/16 Chris G. Davis"

/** \file tests/t0007/basic.c
 *  \brief
 *  \author Chris G. Davis 
 *
*/
#include <stdio.h>
#include "lous/lous.h"
#include "lous/logging.h"
#include "lous/fsm.h"

void callback(fsm_t* fsm, char* tag, char* state) {
  //printf("Callback: Tag: %s State: %s\n", tag, state);

  if( !strcmp(state, "cleanup")) {
    fsm_set_trigger(fsm, tag, "shutdown", 0 );
  }
}

fsm_t* fsm_init() {
  fsm_t* fsm = NULL;
  fsm_state_t* state = NULL;
  fsm_trigger_t* trigger;
  fsm_event_t* event;

  state = fsm_state_alloc("unitialized", NULL, 0, 0);

  fsm = fsm_alloc(state, callback);

  trigger = fsm_trigger_alloc("initialization", 0);
  fsm_trigger_add(fsm, trigger);
  trigger = fsm_trigger_alloc("startup", 0);
  fsm_trigger_add(fsm, trigger);
  trigger = fsm_trigger_alloc("ready", 0);
  fsm_trigger_add(fsm, trigger);
  trigger = fsm_trigger_alloc("shutdown", 0);
  fsm_trigger_add(fsm, trigger);
  trigger = fsm_trigger_alloc("timeout", 0);
  fsm_trigger_add(fsm, trigger);

  /* now, fixup the default state */
  event = fsm_event_alloc("to_shutdown", "cleanup");
  fsm_event_add_param(event, "shutdown", 1);
  fsm_state_add_event(state, event);

  event = fsm_event_alloc("to_initialized", "initialized");
  fsm_event_add_param(event, "initialization", 1);
  fsm_state_add_event(state, event);

  state = fsm_state_alloc("initialized", NULL, 0, 0);
  event = fsm_event_alloc("to_shutdown", "cleanup");
  fsm_event_add_param(event, "shutdown", 1);
  fsm_state_add_event(state, event);
  event = fsm_event_alloc("start_process", "prepare");
  fsm_event_add_param(event, "startup", 1);
  fsm_state_add_event(state, event);
  fsm_state_add(fsm, state);

  state = fsm_state_alloc("prepare", NULL, 0, 0);
  event = fsm_event_alloc("to_shutdown", "cleanup");
  fsm_event_add_param(event, "shutdown", 1);
  fsm_state_add_event(state, event);
  event = fsm_event_alloc("part_way", "trigger_clear");
  fsm_event_add_param(event, "ready", 1);
  fsm_state_add_event(state, event);
  fsm_state_add(fsm, state);

  state = fsm_state_alloc("trigger_clear", NULL, 0, 0);
  event = fsm_event_alloc("to_shutdown", "cleanup");
  fsm_event_add_param(event, "shutdown", 1);
  fsm_state_add_event(state, event);
  event = fsm_event_alloc("finished_prep", "running");
  fsm_event_add_param(event, "startup", 0);
  fsm_event_add_param(event, "ready", 0);
  fsm_state_add_event(state, event);
  fsm_state_add(fsm, state);

  state = fsm_state_alloc("running", NULL, FSM_STATE_TIMEOUT, 10);
  event = fsm_event_alloc("to_shutdown", "cleanup");
  fsm_event_add_param(event, "shutdown", 1);
  fsm_state_add_event(state, event);
  event = fsm_event_alloc("timeout", "running_timeout");
  fsm_event_add_param(event, "timeout", 1);
  fsm_state_add_event(state, event);
  fsm_state_add(fsm, state);

  state = fsm_state_alloc("running_timeout", NULL, 0, 0);
  event = fsm_event_alloc("clear_timeout", "cleanup");
  fsm_event_add_param(event, "timeout", 0);
  fsm_state_add_event(state, event);
  fsm_state_add(fsm, state);

  state = fsm_state_alloc("cleanup", NULL, 0, 0);
  event = fsm_event_alloc("clear_timeout", "initialized");
  fsm_state_add_event(state, event);
  fsm_state_add(fsm, state);
  
  return fsm;
}

void run_step(fsm_t* fsm, fsm_user_t* user) {
  //printf("Step: %p %p\n", fsm, user);
  //printf("========================================\n");
  fsm_run(fsm);

  //fsm_user_display(user);
}

int main() {
  fsm_t* fsm;
  s32_t retval;
  fsm_user_t* user;
  s32_t i;

  LOUS_Debug_Init(LOG_STDERR | LOG_NODATE | LOG_NOTIME,0);
  LOUS_SetDebug(LD_func, LL_veryverbose);
  LOUS_SetDebug(LD_debug, LL_veryverbose);
  
  fsm = fsm_init();

  retval = fsm_lock(fsm);

  fsm_display_fsm(fsm, 0);
    
  for( i = 0; i < 1000; i++ ) {
    user = fsm_user_alloc(fsm, "me");
    
    run_step(fsm, user);
    fsm_user_set_trigger(user, "initialization", 1);
    run_step(fsm, user);
    run_step(fsm, user);
    fsm_user_set_trigger(user, "startup", 1 );
    run_step(fsm, user);
    run_step(fsm, user);
    fsm_user_set_trigger(user, "ready", 1 );
    run_step(fsm, user);
    run_step(fsm, user);
    fsm_user_set_trigger(user, "startup", 0 );
    fsm_user_set_trigger(user, "ready", 0);
    run_step(fsm, user);
    run_step(fsm, user);
    fsm_user_set_trigger(user, "shutdown", 1);
    run_step(fsm, user);
    run_step(fsm, user);
    
    fsm_user_destroy(user);
    
  }

  fsm_display_fsm(fsm, 0);

  return 0;
}

